// This script defines an object holding various functions that override different parts of the
// dojo framework for better usability inside RPE.  

// setup namespace objects
if(!com) {
	var com = {};
}
if(!com.ibm) {
	com.ibm = {};
}
if(!com.ibm.rpe) {
	com.ibm.rpe = {};
}

if(!com.ibm.rpe.DojoFrameworkOverrides) {
	com.ibm.rpe.DojoFrameworkOverrides = function() {}
}

com.ibm.rpe.DojoFrameworkOverrides.parserModuleName = '{0}';
com.ibm.rpe.DojoFrameworkOverrides.savedParseOnLoadState = true;
com.ibm.rpe.DojoFrameworkOverrides.savedDojoConfig = null;
com.ibm.rpe.DojoFrameworkOverrides.dojoVersion = '';
com.ibm.rpe.DojoFrameworkOverrides.usingAsyncLoader = false;
com.ibm.rpe.DojoFrameworkOverrides.foundDojoObject = false;
com.ibm.rpe.DojoFrameworkOverrides.installedOnloadHandler = false;
com.ibm.rpe.DojoFrameworkOverrides.ranRequireAndCreateOverrides = false;



// The main function that drives the override process.
// This function is called repeatedly by a number of script nodes that are injected into
// the source page after every original script element.  This allows us to check on the 
// existence of dojo incrementally as the page loads and step in with our overrides as early
// as possible.  The function nulls itself out once the overrides are installed.
//
// The actual work to override different pieces of the framework is further broken down into 
// smaller chunks and called when appropriate.  This allows us to uniformly  support multiple 
// Dojo versions (which may involve alternate base classes) and multiple loaders (including 
// the AMD loader).
com.ibm.rpe.DojoFrameworkOverrides.applyOverrides = function() {
	// first try to sniff out the dojo config object and modify some of its values.
	// if we don't find it yet that's fine - it will eventually be defined when some subsequent 
	// script element that we injected is run later on.  if we never see a dojo config at all 
	// then this is not actually a dojo page and we'll bypass any of the override code that 
	// would cause extra JS loading
	if(com.ibm.rpe.DojoFrameworkOverrides.tweakDojoConfig()) {
		// we've found and tweaked the dojo config object to turn off parse-on-load

		// check for use of an AMD loader - this has implications for how the overridden code
		// is installed so we'll remember this state
		com.ibm.rpe.DojoFrameworkOverrides.checkForAsyncLoader();

		if (!com.ibm.rpe.DojoFrameworkOverrides.usingAsyncLoader && (typeof dojo === 'undefined')) {
			// If asynchronous loading is not used, wait for dojo to be defined
			return;
		}

		var executeRequireAndCreateOverrides = function() {
			// install a wrapper for dojo.require() that tolerates missing modules.
			// this prevents typos from blowing up the module-loading process and leaving widget types undefined
			com.ibm.rpe.DojoFrameworkOverrides.installDojoRequireWrapper();
			
			// install a wrapper for the widget creation process which automatically copies RPE's internal
			// editor IDs from the source DOM nodes to corresponding widgets.  this allows editor selection to work.
			com.ibm.rpe.DojoFrameworkOverrides.installWidgetCreateWrapper();

			com.ibm.rpe.DojoFrameworkOverrides.ranRequireAndCreateOverrides = true;
		}

		if(!com.ibm.rpe.DojoFrameworkOverrides.installedOnloadHandler) {
			// now set up our override logic to run when the page is ready
			var executeOverride = function() {
				try {
					
					// if we haven't already run the "immediately" overrides, do so now
					if(!com.ibm.rpe.DojoFrameworkOverrides.ranRequireAndCreateOverrides) {
						executeRequireAndCreateOverrides();
					}
					
					// install a wrapper for the dojo parser to make it more fault-tolerant.  then, if parseOnLoad was
					// originally true, invoke the parser ourselves after all the overrides are installed
					com.ibm.rpe.DojoFrameworkOverrides.installParserWrapperAndInvokeParse();

					// turn off 3d translations, as they end up above the design overlay
					com.ibm.rpe.DojoFrameworkOverrides.disableTranslate3d();

				} catch(e) {
					//log('exception installing overrides: '+e.message);
				}
			};
			
			// we need to guarantee that we run the overrides after other scripts may have also required module imports
			if(window.addEventListener) {
				window.addEventListener('load', executeOverride, false);
			} else if(window.attachEvent) {
				// IE
				window.attachEvent('onload', executeOverride);
			} else {
				// if all else fails, do it the ancient way
				var func = window.onload;
				window.onload = function() {
					func && func();
					executeOverride();
				}
			}
			
			com.ibm.rpe.DojoFrameworkOverrides.installedOnloadHandler = true;
		}

		if(typeof dojo !== 'undefined') {
			executeRequireAndCreateOverrides();
			
			// change this function into a no-op so it doesn't take any time on subsequent calls
			com.ibm.rpe.DojoFrameworkOverrides.applyOverrides = function() {};
		}
		
	}
}

com.ibm.rpe.DojoFrameworkOverrides.tweakDojoConfig = function() {
	var djConfigObject = null;

	if (!com.ibm.rpe.DojoFrameworkOverrides.savedDojoConfig) {
		// note: order matters in this sequence of checks - we will stop after the first config object is found.
		// if dojo.config is defined already that means the dojo bootstrap may already have consumed djConfig or dojoConfig, so it's too late to tweak those
	
		if(!(typeof dojo === 'undefined') && dojo.config) {  	
			// "dojo.config"
			//log("dojo.config: "+dojo.config);
			djConfigObject = dojo.config;
		} else if(!(typeof require === 'undefined') && require.rawConfig) {
			// "require.rawConfig" (for AMD loader)
			//log('rawConfig: '+require.rawConfig);
			djConfigObject = require.rawConfig;
		} else if(!(typeof djConfig === 'undefined')) {
			// "djConfig" object
			//log('djConfig: '+djConfig);
			djConfigObject = djConfig;
		} else if(!(typeof dojoConfig === 'undefined')) {
			// "dojoConfig" object
			//log('dojoConfig: '+dojoConfig);
			djConfigObject = dojoConfig;
		} 

		// if we've found the config object, modify some of its values
		// we'll continue to ignore it though until parseOnLoad is a defined property - this means we'll skip past 
		if((djConfigObject != null) && djConfigObject.parseOnLoad) {
			// remember the original state of parseOnLoad and then turn it off - this gives us time to install our overrides before invoking the parser ourselves
			com.ibm.rpe.DojoFrameworkOverrides.savedParseOnLoadState = djConfigObject.parseOnLoad;
			djConfigObject.parseOnLoad = false;
			
			// always turn off debug in the editor environment
			djConfigObject.isDebug = false;
			djConfigObject.debugAtAllCosts = false;

			// save the config object
			com.ibm.rpe.DojoFrameworkOverrides.savedDojoConfig = djConfigObject;

			// returning true tells our calling script that the dojo config has been found and modified
			return true;
		}
		return false;
	}
	return true;
	
}

// Disables 3d translate effects due to their poor interaction with the RPE overlay 
com.ibm.rpe.DojoFrameworkOverrides.disableTranslate3d = function() {
	if (!(typeof dojox === 'undefined') && dojox.mobile) {
		dojox.mobile.hasTranslate3d = false;
	}
	if (typeof require !== 'undefined') {
		// Forces the translate3d feature to not be supported
		require(['dojo/has'], function(has) { has.add('translate3d', false, false, true); })
	}
	
}

com.ibm.rpe.DojoFrameworkOverrides.checkForAsyncLoader = function() {
	if(!(typeof require === 'undefined') && com.ibm.rpe.DojoFrameworkOverrides.savedDojoConfig.async) {
		// we're using the AMD loader - remember this in the static field
		com.ibm.rpe.DojoFrameworkOverrides.usingAsyncLoader = true;
	}
}

com.ibm.rpe.DojoFrameworkOverrides.installDojoRequireWrapper = function() {
	var doWrapping = function() {
		var rpe_dojo_wrap_dojorequire = dojo.require;
		dojo.require = function() {
			try {
				return rpe_dojo_wrap_dojorequire.apply(this, arguments);
			} catch (e) {
				//if(arguments[0]) {
				//	log("dojo.require() failed for '"+arguments[0]+"':\n"+e.message);
				//}
			}
		}
	};

	if(!(typeof dojo === 'undefined')) {
		if(!(typeof(dojo.require) === 'undefined')) {
			doWrapping();
		}
	}
	
	// also wrap require() and the user-defined function passed to it
	if(!(typeof require === 'undefined')) {
		realRequire = require;
		require = function() {
			// we especially need to wrap the user's callback function because exceptions in the callback may cause dojo not to fire ready(), which
			// prevents parsing the page for widgets - note that not all calls to require have a callback though
			try {
				var funcIndex = arguments.length-1;
				if(arguments[funcIndex] && (typeof arguments[funcIndex] === 'function')) {
					var originalUserFunction = arguments[funcIndex];
					arguments[funcIndex] = function() {
						try {
							originalUserFunction.apply(this, arguments);
						} catch (e) {
						}
					}
				}
				
				// now run the real require() using the modified args
				realRequire.apply(this, arguments);
			} catch (e) {
			}
		}
	}
	
}

com.ibm.rpe.DojoFrameworkOverrides.installWidgetCreateWrapper = function() {
	// if we're using the async loader that means Dojo 1.7+, so WidgetBase is always our base class
	if(com.ibm.rpe.DojoFrameworkOverrides.usingAsyncLoader) {
		require(["dojo", "dojo/ready", "dijit/_WidgetBase"], 
				function(dojo, ready, widgetBase) {
					ready(function() {
						com.ibm.rpe.DojoFrameworkOverrides.doWidgetCreateOverride();
					});
				}
		);
	} else {
		// we're using the traditional dojo.require method, so ask for both types of widget base classes and apply the override after they're loaded
		dojo.require('dijit._Widget');
		dojo.require('dijit._WidgetBase');  // only in Dojo 1.6+

		dojo.addOnLoad(function() {
			com.ibm.rpe.DojoFrameworkOverrides.doWidgetCreateOverride();
		});
	}
}

com.ibm.rpe.DojoFrameworkOverrides.doWidgetCreateOverride = function() {
	if(dijit._WidgetBase) {
		// Dojo 1.6 introduced a new (lower) base class so we'll wrap that one if available
		dijit._WidgetBase.prototype.postscript = function(params, srcNodeRef) {
			var id = com.ibm.rpe.Editor.getNodeId(srcNodeRef);
			try {
				this.create(params, srcNodeRef);
			} catch (e) {		
				//log('exception in create() for: '+srcNodeRef+'\r\n'+e.message);
			}
			if(id) {
				com.ibm.rpe.Editor.setNodeId(this.domNode, id);
			}
		}
		
		// also wrap startup() so child startup failures don't take out an entire subtree
		var origStartup = dijit._WidgetBase.prototype.startup;
		dijit._WidgetBase.prototype.startup = function() {
			try {
				origStartup.apply(this, arguments);
			} catch (e) {
			}
		}
	} else if(dijit._Widget) {
		// pre-1.6 base class needs to be used
		dijit._Widget.prototype.postscript = function(params, srcNodeRef) {
			var id = com.ibm.rpe.Editor.getNodeId(srcNodeRef);
			try {
				this.create(params, srcNodeRef);
			} catch (e) {		
				//log('exception in create() for: '+srcNodeRef+'\r\n'+e.message);
			}
			if(id) {
				com.ibm.rpe.Editor.setNodeId(this.domNode, id);
			}
		}
	}
}


com.ibm.rpe.DojoFrameworkOverrides.installParserWrapperAndInvokeParse = function() {
	if(com.ibm.rpe.DojoFrameworkOverrides.usingAsyncLoader) {
		// load the parser module and set up the override in a callback
		var parserName = com.ibm.rpe.DojoFrameworkOverrides.parserModuleName.replace(/\./g, '/');

		// we also need to require dijit/dijit because some mobile scenarios don't implicitly pull it in
		// and we need it for things like dijit.getEnclosingWidget() in various tooling JS during user interactions
		require(["dojo/ready", parserName, "dijit/dijit"], 
		function(ready) {
			ready(function() {
				// install the parser override
				try {
					com.ibm.rpe.DojoFrameworkOverrides.redefineParserCode();
				} catch (e) {
					// if for any reason we couldn't install the parser hacks, we still need to run the parse below
				}
				
				// invoke the parser - this has to go into another ready() so that it doesn't run 
				// until after other require calls in the page have loaded the necessary widget types
				require(["dojo/ready"], 
						function(ready) {
							ready(function() {
								if(com.ibm.rpe.DojoFrameworkOverrides.savedParseOnLoadState) {
									if(com.ibm.rpe.DojoFrameworkOverrides.parserModuleName == 'dojo.parser') {
										dojo.parser.parse();
									} else if(com.ibm.rpe.DojoFrameworkOverrides.parserModuleName == 'dojox.mobile.parser') {
										dojox.mobile.parser.parse();
									}
								}
							});
						
				});	
			});
		});
	} else {
		// request the parser module
		dojo.require(com.ibm.rpe.DojoFrameworkOverrides.parserModuleName);

		// we also need to require dijit.dijit because some mobile scenarios don't implicitly pull it in
		// and we need it for things like dijit.getEnclosingWidget() in various tooling JS during user interactions
		// (pre-1.7 mobile widget type seem to implicitly require it from somewhere in the type hierarchy so 
		// this may be less critical than the require() call above, but just to be safe...)
		dojo.require("dijit.dijit");
		
		dojo.addOnLoad(function() {
			// install the parser override after the module is loaded
			try {
				com.ibm.rpe.DojoFrameworkOverrides.redefineParserCode();
			} catch (e) {
				// if for any reason we couldn't install the parser hacks, we still need to run the parse below
			}
			
			// invoke the parser
			if(com.ibm.rpe.DojoFrameworkOverrides.savedParseOnLoadState) {
				dojo.parser.parse();
			}
		});
	}
}

com.ibm.rpe.DojoFrameworkOverrides.redefineParserCode = function() {
	// Currently the Dojo parser tweaks consist of replacing the parser.instantiate() function and adding a try/catch inside
	// a forEach loop so that broken widget markup won't take down the rest of the page.  There are different function
	// bodies in use depending on the Dojo version and the parser being used.  Dojo 1.7 made substantial changes to the 
	// parsers so currently anything at level Dojo 1.6 and below uses a modified copy of the 1.6 parser, and 1.7 pages 
	// have a special 1.7-derived version instead.  Mobile pages use the "dojox.mobile.parser" module instead of the regular 
	// "dojo.parser", so we'll branch on the module name as well.  This results in 4 different code paths.	
	// 
	// At some point in the future this approach may change somewhat, especially since comments in the parser code 
	// indicate the "instantiate" function will go away in Dojo 2.0.  At that point it should just be a matter of adding
	// additional branches below and replacing different protective try/catch code (assuming that's still necessary).
	
	// capture the dojo version
	var version = dojo.version;
	
	// now branch on the parser module name
	if('dojo.parser' == com.ibm.rpe.DojoFrameworkOverrides.parserModuleName) {
		// the "regular" dojo parser is being used.  now check the dojo version to figure out which override we'll use
		if(version && version.major && version.minor && (version.major == 1) && (version.minor < 7)) {
			// we'll install a modified version of the Dojo 1.6 parser since this is compatible with anything 1.6-and-below
			com.ibm.rpe.DojoFrameworkOverrides.redefineDefault16Parser();
		} else if(version && version.major && version.minor && !(typeof(version.patch) === 'undefined') && (version.major == 1) && (version.minor == 7) && (version.patch < 2)) {
			//SP - the null-check above for version.patch for some reason failed when just checking "&& version.patch" like the rest of the variables, so it needs to be explicitly type-checked
			com.ibm.rpe.DojoFrameworkOverrides.redefineDefault17Parser();
		} else {
			// for 1.7.2 and above...
			com.ibm.rpe.DojoFrameworkOverrides.redefineDefault172Parser();
		}
	} else if('dojox.mobile.parser' == com.ibm.rpe.DojoFrameworkOverrides.parserModuleName) {
		// the dojox.mobile version of the parser is in use - it uses different override code than the regular parser
		
		// we need to further distinguish the Dojo version that's being used to use different overrides, 
		// similar to what we do for the regular parser above.  this is necessary because dojo 1.7 changed the
		// function signature so the internal function we're overriding is not backwards-compatible
		if(version && version.major && version.minor && (version.major == 1) && (version.minor < 7)) {
			// we'll install a modified version of the Dojo 1.6 Mobile parser
			com.ibm.rpe.DojoFrameworkOverrides.redefineMobile16Parser();
		} else if(version && version.major && version.minor && !(typeof(version.patch) === 'undefined') && (version.major == 1) && (version.minor == 7) && (version.patch < 2)) {
			//SP - the null-check above for version.patch for some reason failed when just checking "&& version.patch" like the rest of the variables, so it needs to be explicitly type-checked
			// we'll use a modified version of the Dojo 1.7 Mobile parser
			com.ibm.rpe.DojoFrameworkOverrides.redefineMobile17Parser();
		} else {
			// for anything newer, we need the 1.7.2-based parser
			com.ibm.rpe.DojoFrameworkOverrides.redefineMobile172Parser();
		}
	}
}

com.ibm.rpe.DojoFrameworkOverrides.redefineDefault16Parser = function() {
	// note: this is a modified copy of the dojo 1.6 parser.js code
	// because of scoping rules, we need to redeclare some of the helper functions inside parser.js here also
	
	var d = dojo;

	function val2type(/*Object*/ value){
		// summary:
		//		Returns name of type of given value.

		if(d.isString(value)){ return "string"; }
		if(typeof value == "number"){ return "number"; }
		if(typeof value == "boolean"){ return "boolean"; }
		if(d.isFunction(value)){ return "function"; }
		if(d.isArray(value)){ return "array"; } // typeof [] == "object"
		if(value instanceof Date) { return "date"; } // assume timestamp
		if(value instanceof d._Url){ return "url"; }
		return "object";
	}

	function str2obj(/*String*/ value, /*String*/ type){
		// summary:
		//		Convert given string value to given type
		switch(type){
			case "string":
				return value;
			case "number":
				return value.length ? Number(value) : NaN;
			case "boolean":
				// for checked/disabled value might be "" or "checked".	 interpret as true.
				return typeof value == "boolean" ? value : !(value.toLowerCase()=="false");
			case "function":
				if(d.isFunction(value)){
					// IE gives us a function, even when we say something like onClick="foo"
					// (in which case it gives us an invalid function "function(){ foo }").
					//	Therefore, convert to string
					value=value.toString();
					value=d.trim(value.substring(value.indexOf('{')+1, value.length-1));
				}
				try{
					if(value === "" || value.search(/[^\w\.]+/i) != -1){
						// The user has specified some text for a function like "return x+5"
						return new Function(value);
					}else{
						// The user has specified the name of a function like "myOnClick"
						// or a single word function "return"
						return d.getObject(value, false) || new Function(value);
					}
				}catch(e){ return new Function(); }
			case "array":
				return value ? value.split(/\s*,\s*/) : [];
			case "date":
				switch(value){
					case "": return new Date("");	// the NaN of dates
					case "now": return new Date();	// current date
					default: return d.date.stamp.fromISOString(value);
				}
			case "url":
				return d.baseUrl + value;
			default:
				return d.fromJson(value);
		}
	}

	var dummyClass = {}, instanceClasses = {
		// map from fully qualified name (like "dijit.Button") to structure like
		// { cls: dijit.Button, params: {label: "string", disabled: "boolean"} }
	};

	// Widgets like BorderContainer add properties to _Widget via dojo.extend().
	// If BorderContainer is loaded after _Widget's parameter list has been cached,
	// we need to refresh that parameter list (for _Widget and all widgets that extend _Widget).
	// TODO: remove this in 2.0, when we stop caching parameters.
	d.connect(d, "extend", function(){
		instanceClasses = {};
	});

	function getProtoInfo(cls, params){
		// cls: A prototype
		//		The prototype of the class to check props on
		// params: Object
		//		The parameters object to mix found parameters onto.
		for(var name in cls){
			if(name.charAt(0)=="_"){ continue; }	// skip internal properties
			if(name in dummyClass){ continue; }		// skip "constructor" and "toString"
			params[name] = val2type(cls[name]);
		}
		return params;
	}

	function getClassInfo(/*String*/ className, /*Boolean*/ skipParamsLookup){
		// summary:
		//		Maps a widget name string like "dijit.form.Button" to the widget constructor itself,
		//		and a list of that widget's parameters and their types
		// className:
		//		fully qualified name (like "dijit.form.Button")
		// returns:
		//		structure like
		//			{
		//				cls: dijit.Button,
		//				params: { label: "string", disabled: "boolean"}
		//			}

		var c = instanceClasses[className];
		if(!c){
			// get pointer to widget class
			var cls = d.getObject(className), params = null;
			if(!cls){ return null; }		// class not defined [yet]
			if(!skipParamsLookup){ // from fastpath, we don't need to lookup the attrs on the proto because they are explicit
				params = getProtoInfo(cls.prototype, {})
			}
			c = { cls: cls, params: params };
			
		}else if(!skipParamsLookup && !c.params){
			// if we're calling getClassInfo and have a cls proto, but no params info, scan that cls for params now
			// and update the pointer in instanceClasses[className]. This happens when a widget appears in another
			// widget's template which still uses dojoType, but an instance of the widget appears prior with a data-dojo-type,
			// skipping this lookup the first time.
			c.params = getProtoInfo(c.cls.prototype, {});
		}
		
		return c;
	}
	
	dojo.parser.instantiate = function(/* Array */nodes, /* Object? */mixin, /* Object? */args){
		// summary:
		//		Takes array of nodes, and turns them into class instances and
		//		potentially calls a startup method to allow them to connect with
		//		any children.
		// nodes: Array
		//		Array of nodes or objects like
		//	|		{
		//	|			type: "dijit.form.Button",
		//	|			node: DOMNode,
		//	|			scripts: [ ... ],	// array of <script type="dojo/..."> children of node
		//	|			inherited: { ... }	// settings inherited from ancestors like dir, theme, etc.
		//	|		}
		// mixin: Object?
		//		An object that will be mixed in with each node in the array.
		//		Values in the mixin will override values in the node, if they
		//		exist.
		// args: Object?
		//		An object used to hold kwArgs for instantiation.
		//		See parse.args argument for details.
		var thelist = [],
		mixin = mixin||{};
		args = args||{};

		// TODO: for 2.0 default to data-dojo- regardless of scopeName (or maybe scopeName won't exist in 2.0)
		var attrName = (args.scope || d._scopeName) + "Type",	// typically "dojoType"
			attrData = "data-" + (args.scope || d._scopeName) + "-";	// typically "data-dojo-"

		d.forEach(nodes, function(obj){
			try {				// this try/catch added for RPE
				if(!obj){ return; }

				// Get pointers to DOMNode, dojoType string, and clsInfo (metadata about the dojoType), etc.
				var node, type, clsInfo, clazz, scripts, fastpath;
				if(obj.node){
					// new format of nodes[] array, object w/lots of properties pre-computed for me
					node = obj.node;
					type = obj.type;
					fastpath = obj.fastpath;
					clsInfo = obj.clsInfo || (type && getClassInfo(type, fastpath));
					clazz = clsInfo && clsInfo.cls;
					scripts = obj.scripts;
				}else{
					// old (backwards compatible) format of nodes[] array, simple array of DOMNodes. no fastpath/data-dojo-type support here.
					node = obj;
					type = attrName in mixin ? mixin[attrName] : node.getAttribute(attrName);
					clsInfo = type && getClassInfo(type);
					clazz = clsInfo && clsInfo.cls;
					scripts = (clazz && (clazz._noScript || clazz.prototype._noScript) ? [] :
								d.query("> script[type^='dojo/']", node));
				}
				if(!clsInfo){
					throw new Error("Could not load class '" + type);
				}

				// Setup hash to hold parameter settings for this widget.	Start with the parameter
				// settings inherited from ancestors ("dir" and "lang").
				// Inherited setting may later be overridden by explicit settings on node itself.
				var params = {};
					
				if(args.defaults){
					// settings for the document itself (or whatever subtree is being parsed)
					d._mixin(params, args.defaults);
				}
				if(obj.inherited){
					// settings from dir=rtl or lang=... on a node above this node
					d._mixin(params, obj.inherited);
				}
				
				// mix things found in data-dojo-props into the params
				if(fastpath){
					var extra = node.getAttribute(attrData + "props");
					if(extra && extra.length){
						try{
							extra = d.fromJson.call(args.propsThis, "{" + extra + "}");
							d._mixin(params, extra);
						}catch(e){
							// give the user a pointer to their invalid parameters. FIXME: can we kill this in production?
							throw new Error(e.toString() + " in data-dojo-props='" + extra + "'");
						}
					}

					// For the benefit of _Templated, check if node has data-dojo-attach-point/data-dojo-attach-event
					// and mix those in as though they were parameters
					var attachPoint = node.getAttribute(attrData + "attach-point");
					if(attachPoint){
						params.dojoAttachPoint = attachPoint;
					}
					var attachEvent = node.getAttribute(attrData + "attach-event");
					if(attachEvent){
						params.dojoAttachEvent = attachEvent;
					}
					dojo.mixin(params, mixin);
				}else{
					// FIXME: we need something like "deprecateOnce()" to throw dojo.deprecation for something.
					// remove this logic in 2.0
					// read parameters (ie, attributes) specified on DOMNode

					var attributes = node.attributes;

					// clsInfo.params lists expected params like {"checked": "boolean", "n": "number"}
					for(var name in clsInfo.params){
						var item = name in mixin ? { value:mixin[name], specified:true } : attributes.getNamedItem(name);
						if(!item || (!item.specified && (!dojo.isIE || name.toLowerCase()!="value"))){ continue; }
						var value = item.value;
						// Deal with IE quirks for 'class' and 'style'
						switch(name){
						case "class":
							value = "className" in mixin ? mixin.className : node.className;
							break;
						case "style":
							value = "style" in mixin ? mixin.style : (node.style && node.style.cssText); // FIXME: Opera?
						}
						var _type = clsInfo.params[name];
						if(typeof value == "string"){
							params[name] = str2obj(value, _type);
						}else{
							params[name] = value;
						}
					}
				}

				// Process <script type="dojo/*"> script tags
				// <script type="dojo/method" event="foo"> tags are added to params, and passed to
				// the widget on instantiation.
				// <script type="dojo/method"> tags (with no event) are executed after instantiation
				// <script type="dojo/connect" event="foo"> tags are dojo.connected after instantiation
				// note: dojo/* script tags cannot exist in self closing widgets, like <input />
				var connects = [],	// functions to connect after instantiation
					calls = [];		// functions to call after instantiation

				d.forEach(scripts, function(script){
					node.removeChild(script);
					// FIXME: drop event="" support in 2.0. use data-dojo-event="" instead
					var event = (script.getAttribute(attrData + "event") || script.getAttribute("event")),
						type = script.getAttribute("type"),
						nf = d.parser._functionFromScript(script, attrData);
					if(event){
						if(type == "dojo/connect"){
							connects.push({event: event, func: nf});
						}else{
							params[event] = nf;
						}
					}else{
						calls.push(nf);
					}
				});

				var markupFactory = clazz.markupFactory || clazz.prototype && clazz.prototype.markupFactory;
				// create the instance
				var instance = markupFactory ? markupFactory(params, node, clazz) : new clazz(params, node);
				thelist.push(instance);

				// map it to the JS namespace if that makes sense
				// FIXME: in 2.0, drop jsId support. use data-dojo-id instead
				var jsname = (node.getAttribute(attrData + "id") || node.getAttribute("jsId"));
				if(jsname){
					d.setObject(jsname, instance);
				}

				// process connections and startup functions
				d.forEach(connects, function(connect){
					d.connect(instance, connect.event, null, connect.func);
				});
				d.forEach(calls, function(func){
					func.call(instance);
				});
			} catch (e) {
				// this try/catch added for RPE
				//log('error in parse loop: '+e.message);
			}
		});

		// Call startup on each top level instance if it makes sense (as for
		// widgets).  Parent widgets will recursively call startup on their
		// (non-top level) children
		if(!mixin._started){
			// TODO: for 2.0, when old instantiate() API is desupported, store parent-child
			// relationships in the nodes[] array so that no getParent() call is needed.
			// Note that will  require a parse() call from ContentPane setting a param that the
			// ContentPane is the parent widget (so that the parse doesn't call startup() on the
			// ContentPane's children)
			d.forEach(thelist, function(instance){
				if( !args.noStart && instance  &&
					dojo.isFunction(instance.startup) &&
					!instance._started &&
					(!instance.getParent || !instance.getParent())
				){
					instance.startup();
				}
			});
		}
		return thelist;
	};
// end of Dojo 1.6 parser.instantiate()
}

com.ibm.rpe.DojoFrameworkOverrides.redefineDefault17Parser = function() {
	// the Dojo 1.7 override code for the "regular" parser is below - this is derived from the 1.7 beta in the WAS FeP

	var features = {
			// Feature detection for when node.attributes only lists the attributes specified in the markup
			// rather than old IE/quirks behavior where it lists every default value too
			"dom-attributes-explicit": document.createElement("div").attributes.length < 40
		};

	function has(feature){
		return features[feature];
	}
		
	var d = dojo;

	var _nameMap = {
		// Map from widget name (ex: "dijit.form.Button") to structure mapping
		// lowercase version of attribute names to the version in the widget ex:
		//	{
		//		label: "label",
		//		onclick: "onClick"
		//	}
	};
	function getNameMap(proto){
		// summary:
		//		Returns map from lowercase name to attribute name in class, ex: {onclick: "onClick"}
		var map = {};
		for(var name in proto){
			if(name.charAt(0)=="_"){ continue; }	// skip internal properties
			map[name.toLowerCase()] = name;
		}
		return map;
	}
	// Widgets like BorderContainer add properties to _Widget via dojo.extend().
	// If BorderContainer is loaded after _Widget's parameter list has been cached,
	// we need to refresh that parameter list (for _Widget and all widgets that extend _Widget).
	d.connect(d, "extend", function(){
		_nameMap = {};
	});

	// Map from widget name (ex: "dijit.form.Button") to constructor
	var _ctorMap = {};

	dojo.parser.instantiate = function(nodes, mixin, args){
		// summary:
		//		Takes array of nodes, and turns them into class instances and
		//		potentially calls a startup method to allow them to connect with
		//		any children.
		// nodes: Array
		//		Array of nodes or objects like
		//	|		{
		//	|			type: "dijit.form.Button",
		//	|			node: DOMNode,
		//	|			scripts: [ ... ],	// array of <script type="dojo/..."> children of node
		//	|			inherited: { ... }	// settings inherited from ancestors like dir, theme, etc.
		//	|		}
		// mixin: Object?
		//		An object that will be mixed in with each node in the array.
		//		Values in the mixin will override values in the node, if they
		//		exist.
		// args: Object?
		//		An object used to hold kwArgs for instantiation.
		//		See parse.args argument for details.

		var thelist = [],
		mixin = mixin||{};
		args = args||{};

		// Precompute names of special attributes we are looking for
		// TODO: for 2.0 default to data-dojo- regardless of scopeName (or maybe scopeName won't exist in 2.0)
		var dojoType = (args.scope || d._scopeName) + "Type",		// typically "dojoType"
			attrData = "data-" + (args.scope || d._scopeName) + "-",// typically "data-dojo-"
			dataDojoType = attrData + "type",						// typically "data-dojo-type"
			dataDojoProps = attrData + "props",						// typically "data-dojo-props"
			dataDojoAttachPoint = attrData + "attach-point",
			dataDojoAttachEvent = attrData + "attach-event",
			dataDojoId = attrData + "id";

		// And make hash to quickly check if a given attribute is special, and to map the name to something friendly
		var specialAttrs = {};
		dojo.forEach([dataDojoProps, dataDojoType, dojoType, dataDojoId, "jsId", dataDojoAttachPoint,
				dataDojoAttachEvent, "dojoAttachPoint", "dojoAttachEvent", "class", "style"], function(name){
			specialAttrs[name.toLowerCase()] = name.replace(args.scope, "dojo");
		});

		d.forEach(nodes, function(obj){
			try {				// this try/catch added for RPE
				if(!obj){ return; }

				var node = obj.node || obj,
					type = dojoType in mixin ? mixin[dojoType] : obj.node ? obj.type : (node.getAttribute(dataDojoType) || node.getAttribute(dojoType)),
					ctor = _ctorMap[type] || (_ctorMap[type] = dojo.getObject(type));
					proto = ctor && ctor.prototype;
				if(!ctor){
					throw new Error("Could not load class '" + type);
				}

				// Setup hash to hold parameter settings for this widget.	Start with the parameter
				// settings inherited from ancestors ("dir" and "lang").
				// Inherited setting may later be overridden by explicit settings on node itself.
				var params = {};

				if(args.defaults){
					// settings for the document itself (or whatever subtree is being parsed)
					d._mixin(params, args.defaults);
				}
				if(obj.inherited){
					// settings from dir=rtl or lang=... on a node above this node
					d._mixin(params, obj.inherited);
				}

				// Get list of attributes explicitly listed in the markup
				var attributes;
				if(has("dom-attributes-explicit")){
					// Standard path to get list of user specified attributes
					attributes = node.attributes;
				}else{
					// Special path for IE, avoid (sometimes >100) bogus entries in node.attributes
					var clone = /^input$|^img$/i.test(node.nodeName) ? node : node.cloneNode(false),
						attrs = clone.outerHTML.replace(/=[^\s"']+|="[^"]*"|='[^']*'/g, "").replace(/^\s*<[a-zA-Z0-9]*/, "").replace(/>.*$/, "");

					attributes = dojo.map(attrs.split(/\s+/), function(name){
						var lcName = name.toLowerCase();
						return {
							name: name,
							// getAttribute() doesn't work for button.value, returns innerHTML of button.
							// but getAttributeNode().value doesn't work for the form.encType or li.value
							value: (node.nodeName == "LI" && name == "value") || lcName == "enctype" ?
									node.getAttribute(lcName) : node.getAttributeNode(lcName).value,
							specified: true
						};
					});
				}

				// Read in attributes and process them, including data-dojo-props, data-dojo-type,
				// dojoAttachPoint, etc., as well as normal foo=bar attributes.
				var i=0, item;
				while(item = attributes[i++]){
					if(!item || !item.specified){
						continue;
					}

					var name = item.name,
						lcName = name.toLowerCase(),
						value = item.value;

					if(lcName in specialAttrs){
						switch(specialAttrs[lcName]){

						// Data-dojo-props.   Save for later to make sure it overrides direct foo=bar settings
						case "data-dojo-props":
							var extra = value;
							break;

						// data-dojo-id or jsId. TODO: drop jsId in 2.0
						case "data-dojo-id":
						case "jsId":
							var jsname = value;
							break;

						// For the benefit of _Templated
						case "data-dojo-attach-point":
						case "dojoAttachPoint":
							params.dojoAttachPoint = value;
							break;
						case "data-dojo-attach-event":
						case "dojoAttachEvent":
							params.dojoAttachEvent = value;
							break;

						// Special parameter handling needed for IE
						case "class":
							params["class"] = node.className;
							break;
						case "style":
							params["style"] = node.style && node.style.cssText;
							break;
						}
					}else{
						// Normal attribute, ex: value="123"

						// Find attribute in widget corresponding to specified name.
						// May involve case conversion, ex: onclick --> onClick
						if(!(name in proto)){
							var map = (_nameMap[type] || (_nameMap[type] = getNameMap(proto)));
							name = map[lcName] || name;
						}

						// Set params[name] to value, doing type conversion
						if(name in proto){
							switch(typeof proto[name]){
							case "string":
								params[name] = value;
								break;
							case "number":
								params[name] = value.length ? Number(value) : NaN;
								break;
							case "boolean":
								// for checked/disabled value might be "" or "checked".	 interpret as true.
								params[name] = value.toLowerCase() != "false";
								break;
							case "function":
								if(value === "" || value.search(/[^\w\.]+/i) != -1){
									// The user has specified some text for a function like "return x+5"
									params[name] = new Function(value);
								}else{
									// The user has specified the name of a function like "myOnClick"
									// or a single word function "return"
									params[name] = d.getObject(value, false) || new Function(value);
								}
								break;
							default:
								var pVal = proto[name];
								params[name] =
									(pVal && "length" in pVal) ? (value ? value.split(/\s*,\s*/) : []) :	// array
										(pVal instanceof Date) ?
											(value == "" ? new Date("") :	// the NaN of dates
											value == "now" ? new Date() :	// current date
											d.date.stamp.fromISOString(value)) :
									(pVal instanceof d._Url) ? (d.baseUrl + value) :
									d.fromJson(value);
							}
						}else{
							params[name] = value;
						}
					}
				}

				// Mix things found in data-dojo-props into the params, overriding any direct settings
				if(extra){
					try{
						extra = d.fromJson.call(args.propsThis, "{" + extra + "}");
						d._mixin(params, extra);
					}catch(e){
						// give the user a pointer to their invalid parameters. FIXME: can we kill this in production?
						throw new Error(e.toString() + " in data-dojo-props='" + extra + "'");
					}
				}

				// Any parameters specified in "mixin" override everything else.
				d._mixin(params, mixin);

				var scripts = obj.node ? obj.scripts : (ctor && (ctor._noScript || proto._noScript) ? [] :
							d.query("> script[type^='dojo/']", node));

				// Process <script type="dojo/*"> script tags
				// <script type="dojo/method" event="foo"> tags are added to params, and passed to
				// the widget on instantiation.
				// <script type="dojo/method"> tags (with no event) are executed after instantiation
				// <script type="dojo/connect" event="foo"> tags are dojo.connected after instantiation
				// note: dojo/* script tags cannot exist in self closing widgets, like <input />
				var connects = [],	// functions to connect after instantiation
					calls = [];		// functions to call after instantiation

				if(scripts){
					for(i=0; i<scripts.length; i++){
						var script = scripts[i];
						node.removeChild(script);
						// FIXME: drop event="" support in 2.0. use data-dojo-event="" instead
						var event = (script.getAttribute(attrData + "event") || script.getAttribute("event")),
							type = script.getAttribute("type"),
							nf = this._functionFromScript(script, attrData);
						if(event){
							if(type == "dojo/connect"){
								connects.push({event: event, func: nf});
							}else{
								params[event] = nf;
							}
						}else{
							calls.push(nf);
						}
					}
				}

				// create the instance
				var markupFactory = ctor.markupFactory || proto.markupFactory;
				var instance = markupFactory ? markupFactory(params, node, ctor) : new ctor(params, node);
				thelist.push(instance);

				// map it to the JS namespace if that makes sense
				if(jsname){
					d.setObject(jsname, instance);
				}

				// process connections and startup functions
				for(i=0; i<connects.length; i++){
					d.connect(instance, connects[i].event, null, connects[i].func);
				}
				for(i=0; i<calls.length; i++){
					calls[i].call(instance);
				}
			} catch (e) {
				// this try/catch added for RPE
				//log('exception in parse loop: '+e.message);
			}
		}, this);
		
		// Call startup on each top level instance if it makes sense (as for
		// widgets).  Parent widgets will recursively call startup on their
		// (non-top level) children
		if(!mixin._started){
			// TODO: for 2.0, when old instantiate() API is desupported, store parent-child
			// relationships in the nodes[] array so that no getParent() call is needed.
			// Note that will  require a parse() call from ContentPane setting a param that the
			// ContentPane is the parent widget (so that the parse doesn't call startup() on the
			// ContentPane's children)
			d.forEach(thelist, function(instance){
				if( !args.noStart && instance  &&
					dojo.isFunction(instance.startup) &&
					!instance._started &&
					(!instance.getParent || !instance.getParent())
				){
					instance.startup();
				}
			});
		}
		return thelist;
	};
	// end of 1.7 parser
}

com.ibm.rpe.DojoFrameworkOverrides.redefineDefault172Parser = function() {
	require(
			["dojo/_base/kernel", "dojo/_base/lang", "dojo/_base/array", "dojo/_base/config", "dojo/_base/html", "dojo/_base/window", "dojo/_base/url",
			 	"dojo/_base/json", "dojo/aspect", "dojo/date/stamp", "dojo/has", "dojo/query", "dojo/on", "dojo/ready"],
			function(dojo, dlang, darray, config, dhtml, dwindow, _Url, djson, aspect, dates, has, query, don, ready){
		// module:
//				dojo/parser
		// summary:
//				The Dom/Widget parsing package

		new Date("X"); // workaround for #11279, new Date("") == NaN

		if (has("dom")) {
			var form = document.createElement("form");
			// Test if DOMNode.attributes only lists the attributes the user specified, not attributes w/default values.
			has.add("dom-attributes-explicit", form.attributes.length == 0);

			// IE8 will erroneously list a few attributes that weren't specified,
			// but we know to skip them because they have a specified flag which is false
			has.add("dom-attributes-specified-flag", form.attributes.length < 40);

			// Otherwise, it's IE6-7 form.attributes will list hundreds of values, need to do outerHTML instead.
		}

		dojo.parser = new function(){
			// summary:
			//		The Dom/Widget parsing package

			var _nameMap = {
				// Map from widget name (ex: "dijit.form.Button") to structure mapping
				// lowercase version of attribute names to the version in the widget ex:
				//	{
				//		label: "label",
				//		onclick: "onClick"
				//	}
			};
			function getNameMap(proto){
				// summary:
				//		Returns map from lowercase name to attribute name in class, ex: {onclick: "onClick"}
				var map = {};
				for(var name in proto){
					if(name.charAt(0)=="_"){ continue; }	// skip internal properties
					map[name.toLowerCase()] = name;
				}
				return map;
			}
			// Widgets like BorderContainer add properties to _Widget via dojo.extend().
			// If BorderContainer is loaded after _Widget's parameter list has been cached,
			// we need to refresh that parameter list (for _Widget and all widgets that extend _Widget).
			aspect.after(dlang, "extend", function(){
				_nameMap = {};
			}, true);

			// Map from widget name (ex: "dijit.form.Button") to a map of { "list-of-mixins": ctor }
			// if "list-of-mixins" is "__type" this is the raw type without mixins
			var _ctorMap = {};


			function getCtor(type){
				var map = _ctorMap[type] || (_ctorMap[type] = {});
				return map["__type"] || (map["__type"] = (dlang.getObject(type) || require(type)));
			}

			this._functionFromScript = function(script, attrData){
				// summary:
				//		Convert a <script type="dojo/method" args="a, b, c"> ... </script>
				//		into a function
				// script: DOMNode
				//		The <script> DOMNode
				// attrData: String
				//		For HTML5 compliance, searches for attrData + "args" (typically
				//		"data-dojo-args") instead of "args"
				var preamble = "";
				var suffix = "";
				var argsStr = (script.getAttribute(attrData + "args") || script.getAttribute("args"));
				if(argsStr){
					darray.forEach(argsStr.split(/\s*,\s*/), function(part, idx){
						preamble += "var "+part+" = arguments["+idx+"]; ";
					});
				}
				var withStr = script.getAttribute("with");
				if(withStr && withStr.length){
					darray.forEach(withStr.split(/\s*,\s*/), function(part){
						preamble += "with("+part+"){";
						suffix += "}";
					});
				}
				return new Function(preamble+script.innerHTML+suffix);
			};

			this.instantiate = /*====== dojo.parser.instantiate= ======*/ function(nodes, mixin, options) {
				mixin = mixin || {};
				options = options || {};

				var dojoType = (options.scope || dojo._scopeName) + "Type",		// typically "dojoType"
					attrData = "data-" + (options.scope || dojo._scopeName) + "-",// typically "data-dojo-"
					dataDojoType = attrData + "type";						// typically "data-dojo-type"

				var list = [];
				darray.forEach(nodes, function(node){
					var type = dojoType in mixin ? mixin[dojoType] : node.getAttribute(dataDojoType) || node.getAttribute(dojoType);
					if(type){
						list.push({
							node: node,
							"type": type
						});
					}
				});

				// Instantiate the nodes and return the objects
				return this._instantiate(list, mixin, options);
			};

			this._instantiate = /*====== dojo.parser.instantiate= ======*/ function(nodes, mixin, options){

				var thelist = [];

				// Precompute names of special attributes we are looking for
				// TODO: for 2.0 default to data-dojo- regardless of scopeName (or maybe scopeName won't exist in 2.0)
				var dojoType = (options.scope || dojo._scopeName) + "Type",		// typically "dojoType"
					attrData = "data-" + (options.scope || dojo._scopeName) + "-",// typically "data-dojo-"
					dataDojoType = attrData + "type",						// typically "data-dojo-type"
					dataDojoProps = attrData + "props",						// typically "data-dojo-props"
					dataDojoAttachPoint = attrData + "attach-point",
					dataDojoAttachEvent = attrData + "attach-event",
					dataDojoId = attrData + "id",
					dataDojoMixins = attrData + "mixins";

				// And make hash to quickly check if a given attribute is special, and to map the name to something friendly
				var specialAttrs = {};
				darray.forEach([dataDojoProps, dataDojoType, dojoType, dataDojoId, "jsId", dataDojoAttachPoint,
						dataDojoAttachEvent, "dojoAttachPoint", "dojoAttachEvent", "class", "style", dataDojoMixins], function(name){
					specialAttrs[name.toLowerCase()] = name.replace(options.scope, "dojo");
				});

				function extend(type, mixins){
					return type.createSubclass && type.createSubclass(mixins) || type.extend.apply(type, mixins);
				}

				darray.forEach(nodes, function(obj){
					try {
						if(!obj){ return; }

						var node = obj.node,
							type = obj.type,
							mixins = node.getAttribute(dataDojoMixins), ctor;

						if(mixins){
							var map = _ctorMap[type];
							// remove whitespaces
							mixins = mixins.replace(/ /g, "");
							ctor = map && map[mixins];
							if(!ctor){
								// first get ctor for raw type (& create _ctorMap[type] if needed (should not be))
								ctor = getCtor(type);
								// then do the mixin
								ctor = _ctorMap[type][mixins] = extend(ctor, darray.map(mixins.split(","), getCtor));
							}
						}else{
							ctor = getCtor(type);
						}

						var proto = ctor && ctor.prototype;

						// Setup hash to hold parameter settings for this widget.	Start with the parameter
						// settings inherited from ancestors ("dir" and "lang").
						// Inherited setting may later be overridden by explicit settings on node itself.
						var params = {};

						if(options.defaults){
							// settings for the document itself (or whatever subtree is being parsed)
							dlang.mixin(params, options.defaults);
						}
						if(obj.inherited){
							// settings from dir=rtl or lang=... on a node above this node
							dlang.mixin(params, obj.inherited);
						}

						// Get list of attributes explicitly listed in the markup
						var attributes;
						if(has("dom-attributes-explicit")){
							// Standard path to get list of user specified attributes
							attributes = node.attributes;
						}else if(has("dom-attributes-specified-flag")){
							// Special processing needed for IE8, to skip a few faux values in attributes[]
							attributes = darray.filter(node.attributes, function(a){ return a.specified;});
						}else{
							// Special path for IE6-7, avoid (sometimes >100) bogus entries in node.attributes
							var clone = /^input$|^img$/i.test(node.nodeName) ? node : node.cloneNode(false),
								attrs = clone.outerHTML.replace(/=[^\s"']+|="[^"]*"|='[^']*'/g, "").replace(/^\s*<[a-zA-Z0-9]*\s*/, "").replace(/\s*>.*$/, "");

							attributes = darray.map(attrs.split(/\s+/), function(name){
								var lcName = name.toLowerCase();
								return {
									name: name,
									// getAttribute() doesn't work for button.value, returns innerHTML of button.
									// but getAttributeNode().value doesn't work for the form.encType or li.value
									value: (node.nodeName == "LI" && name == "value") || lcName == "enctype" ?
											node.getAttribute(lcName) : node.getAttributeNode(lcName).value
								};
							});
						}

						// Read in attributes and process them, including data-dojo-props, data-dojo-type,
						// dojoAttachPoint, etc., as well as normal foo=bar attributes.
						var i=0, item;
						while(item = attributes[i++]){
							var name = item.name,
								lcName = name.toLowerCase(),
								value = item.value;

							if(lcName in specialAttrs){
								switch(specialAttrs[lcName]){

								// Data-dojo-props.   Save for later to make sure it overrides direct foo=bar settings
								case "data-dojo-props":
									var extra = value;
									break;

								// data-dojo-id or jsId. TODO: drop jsId in 2.0
								case "data-dojo-id":
								case "jsId":
									var jsname = value;
									break;

								// For the benefit of _Templated
								case "data-dojo-attach-point":
								case "dojoAttachPoint":
									params.dojoAttachPoint = value;
									break;
								case "data-dojo-attach-event":
								case "dojoAttachEvent":
									params.dojoAttachEvent = value;
									break;

								// Special parameter handling needed for IE
								case "class":
									params["class"] = node.className;
									break;
								case "style":
									params["style"] = node.style && node.style.cssText;
									break;
								}
							}else{
								// Normal attribute, ex: value="123"

								// Find attribute in widget corresponding to specified name.
								// May involve case conversion, ex: onclick --> onClick
								if(!(name in proto)){
									var map = (_nameMap[type] || (_nameMap[type] = getNameMap(proto)));
									name = map[lcName] || name;
								}

								// Set params[name] to value, doing type conversion
								if(name in proto){
									switch(typeof proto[name]){
									case "string":
										params[name] = value;
										break;
									case "number":
										params[name] = value.length ? Number(value) : NaN;
										break;
									case "boolean":
										// for checked/disabled value might be "" or "checked".	 interpret as true.
										params[name] = value.toLowerCase() != "false";
										break;
									case "function":
										if(value === "" || value.search(/[^\w\.]+/i) != -1){
											// The user has specified some text for a function like "return x+5"
											params[name] = new Function(value);
										}else{
											// The user has specified the name of a function like "myOnClick"
											// or a single word function "return"
											params[name] = dlang.getObject(value, false) || new Function(value);
										}
										break;
									default:
										var pVal = proto[name];
										params[name] =
											(pVal && "length" in pVal) ? (value ? value.split(/\s*,\s*/) : []) :	// array
												(pVal instanceof Date) ?
													(value == "" ? new Date("") :	// the NaN of dates
													value == "now" ? new Date() :	// current date
													dates.fromISOString(value)) :
											(pVal instanceof dojo._Url) ? (dojo.baseUrl + value) :
											djson.fromJson(value);
									}
								}else{
									params[name] = value;
								}
							}
						}

						// Mix things found in data-dojo-props into the params, overriding any direct settings
						if(extra){
							try{
								extra = djson.fromJson.call(options.propsThis, "{" + extra + "}");
								dlang.mixin(params, extra);
							}catch(e){
								// give the user a pointer to their invalid parameters. FIXME: can we kill this in production?
								throw new Error(e.toString() + " in data-dojo-props='" + extra + "'");
							}
						}

						// Any parameters specified in "mixin" override everything else.
						dlang.mixin(params, mixin);

						var scripts = obj.scripts || (ctor && (ctor._noScript || proto._noScript) ? [] :
									query("> script[type^='dojo/']", node));

						// Process <script type="dojo/*"> script tags
						// <script type="dojo/method" event="foo"> tags are added to params, and passed to
						// the widget on instantiation.
						// <script type="dojo/method"> tags (with no event) are executed after instantiation
						// <script type="dojo/connect" data-dojo-event="foo"> tags are dojo.connected after instantiation
						// <script type="dojo/watch" data-dojo-prop="foo"> tags are dojo.watch after instantiation
						// <script type="dojo/on" data-dojo-event="foo"> tags are dojo.on after instantiation
						// note: dojo/* script tags cannot exist in self closing widgets, like <input />
						var connects = [],	// functions to connect after instantiation
							calls = [],		// functions to call after instantiation
							watch = [],  //functions to watch after instantiation
							on = []; //functions to on after instantiation

						if(scripts){
							for(i=0; i<scripts.length; i++){
								var script = scripts[i];
								node.removeChild(script);
								// FIXME: drop event="" support in 2.0. use data-dojo-event="" instead
								var event = (script.getAttribute(attrData + "event") || script.getAttribute("event")),
									prop = script.getAttribute(attrData + "prop"),
									scriptType = script.getAttribute("type"),
									nf = this._functionFromScript(script, attrData);
								if(event){
									if(scriptType == "dojo/connect"){
										connects.push({event: event, func: nf});
									}else if(scriptType == "dojo/on"){
										on.push({event: event, func: nf});
									}else{
										params[event] = nf;
									}
								}else if(scriptType == "dojo/watch"){
									watch.push({prop: prop, func: nf});
								}else{
									calls.push(nf);
								}
							}
						}

						// create the instance
						var markupFactory = ctor.markupFactory || proto.markupFactory;
						var instance = markupFactory ? markupFactory(params, node, ctor) : new ctor(params, node);
						thelist.push(instance);

						// map it to the JS namespace if that makes sense
						if(jsname){
							dlang.setObject(jsname, instance);
						}

						// process connections and startup functions
						for(i=0; i<connects.length; i++){
							aspect.after(instance, connects[i].event, dojo.hitch(instance, connects[i].func), true);
						}
						for(i=0; i<calls.length; i++){
							calls[i].call(instance);
						}
						for(i=0; i<watch.length; i++){
							instance.watch(watch[i].prop, watch[i].func);
						}
						for(i=0; i<on.length; i++){
							don(instance, on[i].event, on[i].func);
						}
					
					} catch (e) {
						// this try/catch added for RPE
						//log('exception in parser._instantiate: '+e.message);
					}
				}, this);

				// Call startup on each top level instance if it makes sense (as for
				// widgets).  Parent widgets will recursively call startup on their
				// (non-top level) children
				if(!mixin._started){
					darray.forEach(thelist, function(instance){
						if( !options.noStart && instance  &&
							dlang.isFunction(instance.startup) &&
							!instance._started
						){
							instance.startup();
						}
					});
				}
				return thelist;
			};

			this.scan = /*====== dojo.parser.scan= ======*/ function(root, options){
				// Output list
				var list = [];

				var dojoType = (options.scope || dojo._scopeName) + "Type",		// typically "dojoType"
					attrData = "data-" + (options.scope || dojo._scopeName) + "-",	// typically "data-dojo-"
					dataDojoType = attrData + "type",						// typically "data-dojo-type"
					dataDojoTextDir = attrData + "textdir";					// typically "data-dojo-textdir"

				// Info on DOMNode currently being processed
				var node = root.firstChild;

				// Info on parent of DOMNode currently being processed
				//	- inherited: dir, lang, and textDir setting of parent, or inherited by parent
				//	- parent: pointer to identical structure for my parent (or null if no parent)
				//	- scripts: if specified, collects <script type="dojo/..."> type nodes from children
				var inherited = options.inherited;
				if(!inherited){
					function findAncestorAttr(node, attr){
						return (node.getAttribute && node.getAttribute(attr)) ||
							(node !== dwindow.doc && node !== dwindow.doc.documentElement && node.parentNode ? findAncestorAttr(node.parentNode, attr) : null);
					}
					inherited = {
						dir: findAncestorAttr(root, "dir"),
						lang: findAncestorAttr(root, "lang"),
						textDir: findAncestorAttr(root, dataDojoTextDir)
					};
					for(var key in inherited){
						if(!inherited[key]){ delete inherited[key]; }
					}
				}
				var parent = {
					inherited: inherited
				};

				// For collecting <script type="dojo/..."> type nodes (when null, we don't need to collect)
				var scripts;

				// when true, only look for <script type="dojo/..."> tags, and don't recurse to children
				var scriptsOnly;

				function getEffective(parent){
					// summary:
					//		Get effective dir, lang, textDir settings for specified obj
					//		(matching "parent" object structure above), and do caching.
					//		Take care not to return null entries.
					if(!parent.inherited){
						parent.inherited = {};
						var node = parent.node,
							grandparent = getEffective(parent.parent);
						var inherited  = {
							dir: node.getAttribute("dir") || grandparent.dir,
							lang: node.getAttribute("lang") || grandparent.lang,
							textDir: node.getAttribute(dataDojoTextDir) || grandparent.textDir
						};
						for(var key in inherited){
							if(inherited[key]){
								parent.inherited[key] = inherited[key];
							}
						}
					}
					return parent.inherited;
				}

				// DFS on DOM tree, collecting nodes with data-dojo-type specified.
				while(true){
					if(!node){
						// Finished this level, continue to parent's next sibling
						if(!parent || !parent.node){
							break;
						}
						node = parent.node.nextSibling;
						scripts = parent.scripts;
						scriptsOnly = false;
						parent = parent.parent;
						continue;
					}

					if(node.nodeType != 1){
						// Text or comment node, skip to next sibling
						node = node.nextSibling;
						continue;
					}

					if(scripts && node.nodeName.toLowerCase() == "script"){
						// Save <script type="dojo/..."> for parent, then continue to next sibling
						type = node.getAttribute("type");
						if(type && /^dojo\/\w/i.test(type)){
							scripts.push(node);
						}
						node = node.nextSibling;
						continue;
					}
					if(scriptsOnly){
						node = node.nextSibling;
						continue;
					}

					// Check for data-dojo-type attribute, fallback to backward compatible dojoType
					var type = node.getAttribute(dataDojoType) || node.getAttribute(dojoType);

					// Short circuit for leaf nodes containing nothing [but text]
					var firstChild = node.firstChild;
					if(!type && (!firstChild || (firstChild.nodeType == 3 && !firstChild.nextSibling))){
						node = node.nextSibling;
						continue;
					}

					// Setup data structure to save info on current node for when we return from processing descendant nodes
					var current = {
						node: node,
						scripts: scripts,
						parent: parent
					};

					// If dojoType/data-dojo-type specified, add to output array of nodes to instantiate
					// Note: won't find classes declared via dojo.Declaration, so use try/catch to avoid throw from require()
					// We don't care yet about mixins ctors, we check script stop only on main class
					var ctor;
					try{
						ctor = type && getCtor(type);
					}catch(e){
					}
					var childScripts = ctor && !ctor.prototype._noScript ? [] : null; // <script> nodes that are parent's children
					if(type){
						list.push({
							"type": type,
							node: node,
							scripts: childScripts,
							inherited: getEffective(current) // dir & lang settings for current node, explicit or inherited
						});
					}

					// Recurse, collecting <script type="dojo/..."> children, and also looking for
					// descendant nodes with dojoType specified (unless the widget has the stopParser flag).
					// When finished with children, go to my next sibling.
					node = firstChild;
					scripts = childScripts;
					scriptsOnly = ctor && ctor.prototype.stopParser && !(options.template);
					parent = current;
				}

				return list;
			};

			this.parse = /*====== dojo.parser.parse= ======*/ function(rootNode, options){
				// determine the root node and options based on the passed arguments.
				var root;
				if(!options && rootNode && rootNode.rootNode){
					options = rootNode;
					root = options.rootNode;
				}else if(rootNode && dlang.isObject(rootNode) && !("nodeType" in rootNode)){
					options = rootNode;
				}else{
					root = rootNode;
				}
				root = root ? dhtml.byId(root) : dwindow.body();

				options = options || {};

				// List of all nodes on page w/dojoType specified
				var list = this.scan(root, options);

				// go build the object instances
				var mixin = options.template ? {template: true} : {};
				return this._instantiate(list, mixin, options); // Array
			};
		}();
	});
}

com.ibm.rpe.DojoFrameworkOverrides.redefineMobile16Parser = function() {
	// mobile parser for Dojo 1.6 and below  (this is a modified copy of the 1.6 mobile parser)
	dojox.mobile.parser.instantiate = function(list, defaultParams){
		// summary:
		//		Function for instantiating a list of widget nodes.
		// list:
		//		The list of DOMNodes to walk and instantiate widgets on.
		var ws = [];
		if(list){
			var i, len;
			len = list.length;
			for(i = 0; i < len; i++){
				try {
					var node = list[i];
					var cls = dojo.getObject(dojo.attr(node, "dojoType"));
					var proto = cls.prototype;
					var params = {};

					if(defaultParams){
						for(var name in defaultParams){
							params[name] = defaultParams[name];
						}
					}
					for(var prop in proto){
						var val = dojo.attr(node, prop);
						if(!val){ continue; }
						if(typeof proto[prop] == "string"){
							params[prop] = val;
						}else if(typeof proto[prop] == "number"){
							params[prop] = val - 0;
						}else if(typeof proto[prop] == "boolean"){
							params[prop] = (val != "false");
						}else if(typeof proto[prop] == "object"){
							params[prop] = eval("(" + val + ")");
						}
					}
					params["class"] = node.className;
					params["style"] = node.style && node.style.cssText;
					ws.push(new cls(params, node));
				} catch (e) {
					//log('parser error: '+e.message);
				}
			}
			len = ws.length;
			for(i = 0; i < len; i++){
				var w = ws[i];
				w.startup && !w._started && (!w.getParent || !w.getParent()) && w.startup();
			}
		}
		return ws;
	};
	
	// for <= 1.6, we need to manually set up an alias so RPE's parse-on-load works - the 1.7 parser.js does this already
	dojo.parser = dojox.mobile.parser;
}

com.ibm.rpe.DojoFrameworkOverrides.redefineMobile17Parser = function() {
	// mobile parser for Dojo 1.7 (beta)
	dojox.mobile.parser.instantiate = function(/* Array */nodes, /* Object? */mixin, /* Object? */args){
		// summary:
		//		Function for instantiating a list of widget nodes.
		// nodes:
		//		The list of DOMNodes to walk and instantiate widgets on.
		mixin = mixin || {};
		args = args || {};
		var i, ws = [];
		if(nodes){
			for(i = 0; i < nodes.length; i++){
				try {
					var n = nodes[i];
					var cls = dojo.getObject(n.getAttribute("dojoType") || n.getAttribute("data-dojo-type"));
					var proto = cls.prototype;
					var params = {}, prop, v, t;
					dojo._mixin(params, eval('({'+(n.getAttribute("data-dojo-props")||"")+'})'));
					dojo._mixin(params, args.defaults);
					dojo._mixin(params, mixin);
					for(prop in proto){
						v = n.getAttributeNode(prop);
						v = v && v.nodeValue;
						t = typeof proto[prop];
						if(!v && (t !== "boolean" || v !== "")){ continue; }
						if(t === "string"){
							params[prop] = v;
						}else if(t === "number"){
							params[prop] = v - 0;
						}else if(t === "boolean"){
							params[prop] = (v !== "false");
						}else if(t === "object"){
							params[prop] = eval("(" + v + ")");
						}
					}
					params["class"] = n.className;
					params.style = n.style && n.style.cssText;
					v = n.getAttribute("data-dojo-attach-point");
					if(v){ params.dojoAttachPoint = v; }
					v = n.getAttribute("data-dojo-attach-event");
					if(v){ params.dojoAttachEvent = v; }
					var instance = new cls(params, n);
					ws.push(instance);
					var jsId = n.getAttribute("jsId") || n.getAttribute("data-dojo-id");
					if(jsId){
						dojo.setObject(jsId, instance);
					}
				} catch (e) {
					//log('error in parser: '+e.message);
				}
			}
			for(i = 0; i < ws.length; i++){
				var w = ws[i];
				!args.noStart && w.startup && !w._started && (!w.getParent || !w.getParent()) && w.startup();
			}
		}
		return ws;
	};
}

com.ibm.rpe.DojoFrameworkOverrides.redefineMobile172Parser = function() {
	// mobile parser for Dojo 1.7.2
	require(["dojo/_base/kernel", "dojo/_base/array", "dojo/_base/config",	"dojo/_base/lang",	"dojo/_base/window", "dojo/ready" ],
			function(dojo, array, config, lang, win, ready){
		
		var dm = lang.getObject("dojox.mobile", true);

		var parser = new function(){
			// summary:
			//		A lightweight parser.
			// description:
			//		dojox.mobile.parser is an extremely small subset of dojo.parser.
			//		It has no extended features over dojo.parser, so there is no
			//		reason you must use dojox.mobile.parser instead of dojo.parser.
			//		However, if dojox.mobile.parser's capability is enough for your
			//		application, use of it could reduce the total code size.

			var _ctorMap = {};
			var getCtor = function(type, mixins){
				if(typeof(mixins) === "string"){
					var t = type + ":" + mixins.replace(/ /g, "");
					return _ctorMap[t] ||
						(_ctorMap[t] = getCtor(type).createSubclass(array.map(mixins.split(/, */), getCtor)));
				}
				return _ctorMap[type] || (_ctorMap[type] = lang.getObject(type) || require(type));
			};
			var _eval = function(js){ return eval(js); };

			this.instantiate = function(/* Array */nodes, /* Object? */mixin, /* Object? */options){
				// summary:
				//		Function for instantiating a list of widget nodes.
				// nodes:
				//		The list of DOMNodes to walk and instantiate widgets on.
				mixin = mixin || {};
				options = options || {};
				var i, ws = [];
				if(nodes){
					for(i = 0; i < nodes.length; i++){
						try {
							var n = nodes[i],
							type = n._type,
							ctor = getCtor(type, n.getAttribute("data-dojo-mixins")),
							proto = ctor.prototype,
							params = {}, prop, v, t;
							lang.mixin(params, _eval.call(options.propsThis, '({'+(n.getAttribute("data-dojo-props")||"")+'})'));
							lang.mixin(params, options.defaults);
							lang.mixin(params, mixin);
							for(prop in proto){
								v = n.getAttributeNode(prop);
								v = v && v.nodeValue;
								t = typeof proto[prop];
								if(!v && (t !== "boolean" || v !== "")){ continue; }
								if(lang.isArray(proto[prop])){
									params[prop] = v.split(/\s*,\s*/);
								}else if(t === "string"){
									params[prop] = v;
								}else if(t === "number"){
									params[prop] = v - 0;
								}else if(t === "boolean"){
									params[prop] = (v !== "false");
								}else if(t === "object"){
									params[prop] = eval("(" + v + ")");
								}else if(t === "function"){
									params[prop] = lang.getObject(v, false) || new Function(v);
									n.removeAttribute(prop);
								}
							}
							params["class"] = n.className;
							if(!params.style){ params.style = n.style.cssText; }
							v = n.getAttribute("data-dojo-attach-point");
							if(v){ params.dojoAttachPoint = v; }
							v = n.getAttribute("data-dojo-attach-event");
							if(v){ params.dojoAttachEvent = v; }
							var instance = new ctor(params, n);
							ws.push(instance);
							var jsId = n.getAttribute("jsId") || n.getAttribute("data-dojo-id");
							if(jsId){
								lang.setObject(jsId, instance);
							}
						} catch(e) {
							//log('exception in mobile parser.instantiate: '+e.message);
						}
					}
					for(i = 0; i < ws.length; i++){
						var w = ws[i];
						!options.noStart && w.startup && !w._started && w.startup();
					}
				}
				return ws;
			};
		
			this.parse = function(rootNode, options){
				// summary:
				//		Function to handle parsing for widgets in the current document.
				//		It is not as powerful as the full parser, but it will handle basic
				//		use cases fine.
				// rootNode:
				//		The root node in the document to parse from
				if(!rootNode){
					rootNode = win.body();
				}else if(!options && rootNode.rootNode){
					// Case where 'rootNode' is really a params object.
					options = rootNode;
					rootNode = rootNode.rootNode;
				}

				var nodes = rootNode.getElementsByTagName("*");
				var i, j, list = [];
				for(i = 0; i < nodes.length; i++){
					try {
						var n = nodes[i],
							type = (n._type = n.getAttribute("dojoType") || n.getAttribute("data-dojo-type"));
						if(type){
							if(n._skip){
								n._skip = undefined;
								continue;
							}
							if(getCtor(type).prototype.stopParser && !(options && options.template)){
								var arr = n.getElementsByTagName("*");
								for(j = 0; j < arr.length; j++){
									arr[j]._skip = 1;
								}
							}
							list.push(n);
						}
					} catch (e) {
						 //log('exception in mobile parser.parse(): '+e.message);
					}
				}
				var mixin = options && options.template ? {template: true} : null;
				return this.instantiate(list, mixin, options);
			};
		}();
		dm.parser = parser; // for backward compatibility
		dojo.parser = dojo.parser || parser; // in case user application calls dojo.parser
	});
}